// -----------------------------------------------------------------------------
// File: title_screen.ss
// Description: title screen script
// Author: Alexandre Martins <http://opensurge2d.org>
// License: MIT
// -----------------------------------------------------------------------------
using SurgeEngine;
using SurgeEngine.Transform;
using SurgeEngine.Vector2;
using SurgeEngine.Actor;
using SurgeEngine.Player;
using SurgeEngine.Level;
using SurgeEngine.UI.Text;
using SurgeEngine.Video.Screen;
using SurgeEngine.Audio.Sound;
using SurgeEngine.Audio.Music;
using SurgeEngine.Camera;
using SurgeEngine.Web;

// ----------------------------------------------------------------------------
// TITLE SCREEN: SETUP OBJECT
// ----------------------------------------------------------------------------

object "Select Screen"
{
    fader = spawn("Fader");
    menu = spawn("Select Screen - Menu").setFader(fader);
	text = spawn("Select Screen - Text");

    fun constructor()
    {
        fader.fadeTime = 0.5;
        fader.fadeIn();
    }
}

// ----------------------------------------------------------------------------
// GAME LOGO
// ----------------------------------------------------------------------------

object "Select Screen - Text" is "private", "detached", "entity"
{
    transform = Transform();
    label = Text("Select Screen - Menu Item");

    fun constructor()
    {
        transform.position = Vector2(200, 30);
        label.align = "center";
        label.text = "$SELECTSCREEN_TEXT";
        label.maxWidth = Screen.width - 16;
    }
}
// ----------------------------------------------------------------------------
// MENU: BASIC LOGIC
// ----------------------------------------------------------------------------

object "Select Screen - Menu" is "private", "detached", "entity"
{
    transform = Transform();
    itemGroup = spawn("Select Screen - Menu Item Group");
    fader = null;
    selectedMenuItem = null;

    state "main"
    {
    }

    state "fading"
    {
        if(timeout(fader.fadeTime + 0.5)) {
            selectedMenuItem.onEnter();
            state = "done";
        }
    }

    state "done"
    {
    }

    fun onSelect(menuItem)
    {
        //Console.print(menuItem.__name);
        selectedMenuItem = menuItem;
        fader.fadeOut();
        state = "fading";
    }

    fun setFader(obj)
    {
        fader = obj;
        return this;
    }

    fun constructor()
    {
        transform.position = Vector2(Screen.width / 2, Screen.height - 32);
    }
}

object "Select Screen - Menu Item Group" is "detached", "entity"
{
    transform = Transform();
    slide = Sound("samples/slide.wav");
    select = Sound("samples/select.wav");
    spacing = 128; // spacing between menu items
    selectedItem = 0;
    items = [
        spawn("Select Screen - Menu Item - Speedy").setOffset(Vector2.right.scaledBy(0 * spacing)),
        spawn("Select Screen - Menu Item - Zippy").setOffset(Vector2.right.scaledBy(1 * spacing)),
    ];
    swipeTime = 0.5; // in seconds
    timer = 1.0;

    state "main"
    {
        // item select
        input = Player.active.input;
        if(input.buttonPressed("right")) {
            Level.background = "themes/charzippy.bg";
			selectItem(selectedItem + 1);
		}
        else if(input.buttonPressed("left")) {
            Level.background = "themes/charspeedy.bg";
			selectItem(selectedItem - 1);
		}

        // swipe logic
        timer += Time.delta / swipeTime;
        x = Math.smoothstep(transform.localPosition.x, selectedItem * -spacing, timer);
        transform.localPosition = Vector2(Math.round(x), 0);
    }

    fun selectItem(itemNumber)
    {
        if(itemNumber < 0 || itemNumber >= items.length)
            return;

        items[selectedItem].setHighlighted(false);
        items[itemNumber].setHighlighted(true);

        timer = 0.0;
        selectedItem = itemNumber;
        slide.play();
    }

    fun onSelect(menuItem)
    {
        Player.active.input.enabled = false;
        select.play();
        parent.onSelect(menuItem);
    }
}


// ----------------------------------------------------------------------------
// MENU ITEMS
// ----------------------------------------------------------------------------

object "Select Screen - Menu Item - Speedy" is "private", "detached", "entity"
{
    delegate = spawn("Select Screen - Menu Item")
               .setText("$SELECTSCREEN_SPEEDY")
               .setHighlighted(true);

    fun onEnter()
    {
        Level.load("quests/speedyquest.qst");
    }

    fun onSelect()
    {
        parent.onSelect(this);
    }

    fun setHighlighted(highlighted)
    {
        delegate.setHighlighted(highlighted);
        return this;
    }

    fun setOffset(offset)
    {
        delegate.setOffset(offset);
        return this;
    }
}

object "Select Screen - Menu Item - Zippy" is "private", "detached", "entity"
{
    delegate = spawn("Select Screen - Menu Item")
               .setText("$SELECTSCREEN_ZIPPY");

    fun onEnter()
    {
        Level.load("quests/zippyquest.qst");
    }

    fun onSelect()
    {
        parent.onSelect(this);
    }

    fun setHighlighted(highlighted)
    {
        delegate.setHighlighted(highlighted);
        return this;
    }

    fun setOffset(offset)
    {
        delegate.setOffset(offset);
        return this;
    }
}

object "Select Screen - Menu Item" is "private", "detached", "entity"
{
    transform = Transform();
    label = Text("Select Screen - Menu Item");
    text = "";
    highlighted = false;

    state "main"
    {
        if(!highlighted)
            return;

        input = Player.active.input;
        if(input.buttonPressed("fire1") || input.buttonPressed("fire3"))
            parent.onSelect();
    }

    fun setText(txt)
    {
        text = txt;
        label.text = highlighted ? "<color=$COLOR_HIGHLIGHT>" + text + "</color>" : text;
        return this;
    }

    fun setHighlighted(h)
    {
        highlighted = h;
        label.text = highlighted ? "<color=$COLOR_HIGHLIGHT>" + text + "</color>" : text;
        return this;
    }

    fun setOffset(offset)
    {
        transform.localPosition = offset;
        return this;
    }

    fun constructor()
    {
        label.align = "center";
    }
}